#!/usr/bin/python3
#

# For use with this program, setup RS232 of the scope as follows:
# Baudrate: 38400, Hard Flagging: On, Soft Flagging: Off, EOL: LF, Parity: Even, Stop Bits: 1, Delay: 0s.
#
# Setup Hard Copy as follows:
# Layout: Landscape, Format: EPS Image
#
# Connect scope to PC and then run this program. Python 3.x required.
# Hard Copies will be saved in the hard_copies directory located in the same directory as this program.
# EPS files can be converted to PNG using ImageMagick convert:
#
# convert -density 300 THS_latest.eps -rotate 90 -flatten +repage THS_latest_300dpi.png
#
# This will also rotate the image clockwise so that it is properly oriented.
# Play with -density argument to control resolution.
#
# Author: djnz @ eevblog forum


import serial.tools.list_ports
import serial
import datetime
import os
import time

def choosePort():
    # ports = [p.device for p in serial.tools.list_ports.comports()]
    ports = serial.tools.list_ports.comports()
    ports.sort()
    if len(ports) < 1:
        raise RuntimeError("\n\nCould not determine serial port used for connecting to the THS. Is the scope connected?")
    elif len(ports) == 1:
        print("Using " + str(ports[0]))
        return ports[0]
    else:
        print("Multiple ports found. Choose port. Enter index between 0 and " + str(len(ports)-1))
        for index in range(len(ports)):
            print(str(index) + " : " + str(ports[index]))
        chosen = input("Enter your choice...")
        chosen = int(chosen)
        if chosen >= len(ports):
            raise ValueError("\n\nInvalid Choice")
        print("Using " + str(ports[chosen]))
        return ports[chosen]

def checkForTek(port):
    old_timeout = port.timeout
    port.timeout = 10
    port.write(b'*IDN?\n')
    time.sleep(1)
    resp = port.read_all()
    if not b'TEKTRONIX,THS7' in resp:
        port.timeout = old_timeout
        raise RuntimeError("\n\nCould not find THS7 device. Device at " + port.name + " gave unexpected response to *IDN? query.")
    else:
        port.timeout = old_timeout
        print("Found Tek THS7x device: " + str(resp, 'utf8'))



def main():
    chosenport = choosePort().device
    p = serial.Serial(port=chosenport, timeout=5, baudrate=38400, rtscts=True, xonxoff=False, parity=serial.PARITY_EVEN, stopbits=serial.STOPBITS_ONE)
    checkForTek(p)
    p.timeout = None
    p.read_all()
    scopedata = None
    installdir = os.path.dirname(__file__)
    savepath = os.path.join(installdir, 'hard_copies/')

    # Terminators for different Hard Copy types.
    # TIFF image size is 320x240
    # term_tiff = b'1995:04:05 12:00:00\x00'
    # term_DPU411 = b'\x00\x00\x00\x00\x1b\x4a\x10\x0d\x0d\x0a\x0d\x0a'
    # term_DPU412 = b'\x0d\x1b\x32\x0d\x0a\x0d\x0a\x0d\x0a'
    # eps seems to be the best format as eps -> png size allows higher resolutions while maintaining quality
    term_eps = b'\x3a\x20\x31\x0d\x25\x25\x45\x4f\x46\x0d'


    while True:
        print("Waiting for data...")
        scopedata = p.read(1)
        print("Receiving data...")
        scopedata = scopedata + p.read_until(term_eps)
        print("Received " + str(len(scopedata)) + " bytes of data." )

        # timestamp = datetime.datetime.now().isoformat()
        timestamp = datetime.datetime.now().strftime('%Y'"-"'%m'"-"'%d'"_"'%H'"-"'%M'"-"'%S')
        p.read_all()
        filename1 = os.path.join(savepath, "THS_" + timestamp + ".eps")
        myfile = open(filename1, mode='wb')
        myfile.write(scopedata)
        myfile.close()

        filename2 = os.path.join(savepath, "THS_latest" + ".eps")
        myfile = open(filename2, mode='wb')
        myfile.write(scopedata)
        myfile.close()

        print("Saved: ")
        print(filename1)
        print(filename2)
        print("Looping... \n")


main()
